--sourced from Callmore#4947's kart addon Scorekeeper
--ported to vanilla srb2 and heavily modified for use in LifeStuff+ by Xian
--vanilla doesn't allow io.open to write data like kart does
--vanilla also requires a callback function in order to use io.open, kart does not
--io.open does not return the opened file like kart does
--tldr, had to make heavy changes to make this work, including converting to io.openlocal. Least I learned a few things

local FILENAME = "client/lifestuff.txt"
local globalData = {}
local dectimer = 0

local cv_hourstoreset = CV_RegisterVar{
	name = "lsplus_hoursuntilreset",
	defaultvalue = 168,
	flags = CV_NETVAR,
	PossibleValue = CV_Natural
}

-- Here we attempt to open the score file to read the scores into memory.
-- If the file doesent exist, io.open silently fails.
-- io.open needs to be wrapped inside of an assert to error when a file is
--   failed to be opened.

local file = io.openlocal(FILENAME, "a")  --create the file if it doesn't exist, but don't alter it. we use append mode for this

if file then
	print("Found scores data")
else
	print("No scores data found, initializing new savedata")
end
file:close()

local file2 = io.openlocal(FILENAME, "r") 	-- file already exsists, load from it

if file2 then
	print('Loading scores from "lifestuff.txt"...')

	for l in file2:lines() do
		local name, flives, matchreset, deaths, score, tlives, scrap  = string.match(l, "(.*);(.*);(.*);(.*);(.*);(.*);(.*)")

		if name and name != "Sonic" and name != "Tails" then
			globalData[name] = {tonumber(flives), tonumber(matchreset), tonumber(deaths), tonumber(score), tonumber(tlives), tonumber(scrap)}
		end
	end
end
file2:close()

local function getHoursToReset()
	return TICRATE*60*60*cv_hourstoreset.value
end

local function _saveFileFunc()
	-- This function actully opens the file.
	-- If the file failed to open for whatever reason, the assert would catch
	--   it and return back to the pcall that called this function.
	local f = assert(io.openlocal(FILENAME, "w"))
	for pn, s in pairs(globalData) do
		if pn:find(";") then continue end -- reject any names that contain a semicolon, since OOPS HOSTS CAN DO THAT LOL
		if pn == "Sonic" or pn == "Tails" then continue end --reject any default usernames. find a unique name or no saving for you
		f:write(pn, ";", s[1], ";", s[2], ";", s[3], ";", s[4], ";", s[5], ";", s[6], "\n")
	end
	f:close()
end

local function saveFile()
	-- This function is effectivly a wrapper for the real open file function.
	-- The real open file function opens the file through an assert,
	--   if the file failed to open, it would immediatly stop execution and
	--   return here with a false, causing the if block to be ran and to print
	--   the message.
	if not isserver then return end
	print('Saving scores to "scorekeeper.txt"...')
	if not pcall(_saveFileFunc) then
		CONS_Printf(server, "Failed to save file!")
	end
end

local function incdec()
	dectimer = min($+1, getHoursToReset())
end

local hasRanInt = false

local function think()
	if not netgame then return end
	if hasRanInt then
		hasRanInt = false
	end
	
	incdec()
	for p in players.iterate do
		if not p.sk_loaded then
			p.sk_loaded = true
			if globalData[p.name] != nil then --if the server has playerdata for us, load it
				p.fakedisplives = globalData[p.name][1]
				p.fakedispdeathcounter = globalData[p.name][3]
				p.score = globalData[p.name][4]
				p.lives = globalData[p.name][5]
				if p.fakedispsavedscrap >= 0 then
					p.skipscrap = globalData[p.name][6]
				end
			end
		elseif p.name ~= p.sk_lastname then --if our name was changed recently
			if globalData[p.name] ~= nil then --and our new name has data already
				p.fakedisplives = globalData[p.name][1]
				p.fakedispdeathcounter = globalData[p.name][3] --update our info to the new name data
				p.score = globalData[p.name][4]
				p.lives = globalData[p.name][5]
				if p.fakedispsavedscrap >= 0 then
					p.skipscrap = globalData[p.name][6]
				end
			else
				--check if oldname exists
				if globalData[p.sk_lastname] then
					--they exsist, save score to old name and reset
					globalData[p.sk_lastname][1] = p.fakedisplives
					globalData[p.sk_lastname][3] = p.fakedispdeathcounter
					globalData[p.sk_lastname][4] = p.score
					globalData[p.sk_lastname][5] = p.lives
					globalData[p.sk_lastname][6] = p.fakedispsavedscrap
				end
				globalData[p.name] = {3, getHoursToReset(), 0, 0, 3, 0}
				p.fakedisplives = 3
				p.fakedispdeathcounter = 0
				p.score = 0
				p.lives = 3
				if p.fakedispsavedscrap >= 0 then
					p.skipscrap = 40
				end				
			end
		end
		p.sk_lastname = p.name
	end
	--saveFile()
end
addHook("ThinkFrame", think)

local function intThink()
	if not netgame then return end
	incdec()
	if hasRanInt then return end
	hasRanInt = true
	local pingame = {}
	for p in players.iterate do
		if p.name == "Sonic" or p.name == "Tails" then continue end --get out of here with that lame default name
		globalData[p.name] = {p.fakedisplives, getHoursToReset(), p.fakedispdeathcounter, p.score, p.lives, p.fakedispsavedscrap}
		pingame[p.name] = true
	end
	for n, v in pairs(globalData) do
		if pingame[n] then continue end
		if tonumber(v[2]) - dectimer > 0 then
			globalData[n][2] = $ - dectimer
		else
			--they havent played in too long, wipe data
			globalData[n] = nil
		end
	end
	CONS_Printf(server, "saving scores")
	saveFile()
	dectimer = 0
end
addHook("IntermissionThinker", intThink)

local function netvars(net)
	globalData = net($)
	dectimer = net($)
	hasRanInt = net($)
end
addHook("NetVars", netvars)
